/*============================================================================
 * This function is called at the end of each time step, and has a very
 *  general purpose
 *  (i.e. anything that does not have another dedicated user subroutine)
 *============================================================================*/

/* VERS */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2016 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

#include "cs_defs.h"

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/

#include <assert.h>
#include <math.h>

#if defined(HAVE_MPI)
#include <mpi.h>
#endif

/*----------------------------------------------------------------------------
 * PLE library headers
 *----------------------------------------------------------------------------*/

#include <ple_coupling.h>

/*----------------------------------------------------------------------------
 *  Local headers
 *----------------------------------------------------------------------------*/

#include "bft_mem.h"
#include "bft_error.h"
#include "bft_printf.h"

#include "fvm_writer.h"

#include "cs_base.h"
#include "cs_field.h"
#include "cs_field_pointer.h"
#include "cs_field_operator.h"
#include "cs_mesh.h"
#include "cs_mesh_quantities.h"
#include "cs_halo.h"
#include "cs_halo_perio.h"
#include "cs_log.h"
#include "cs_parall.h"
#include "cs_parameters.h"
#include "cs_prototypes.h"
#include "cs_rotation.h"
#include "cs_time_moment.h"
#include "cs_time_step.h"
#include "cs_turbomachinery.h"
#include "cs_selector.h"

#include "cs_post.h"

/*----------------------------------------------------------------------------
 *  Header for the current file
 *----------------------------------------------------------------------------*/

#include "cs_prototypes.h"

/*----------------------------------------------------------------------------*/

BEGIN_C_DECLS

/*----------------------------------------------------------------------------*/
/*!
 * \file cs_user_extra_operations.c
 *
 * \brief This function is called at the end of each time step, and has a very
 * general purpose (i.e. anything that does not have another dedicated
 * user subroutine)
 *
 */
/*----------------------------------------------------------------------------*/

/*============================================================================
 * User function definitions
 *============================================================================*/

/*----------------------------------------------------------------------------*/
/*!
 * \brief This function is called at the end of each time step.
 *
 * It has a very general purpose, although it is recommended to handle
 * mainly postprocessing or data-extraction type operations.
 */
/*----------------------------------------------------------------------------*/

void
cs_user_extra_operations(void)
{
  FILE *file = NULL;
  cs_real_3_t *vel = (cs_real_3_t *)CS_F_(u)->val;
  const cs_real_3_t *cell_cen = (const cs_real_3_t *)cs_glob_mesh_quantities->cell_cen;
  const cs_lnum_t n_cells     = cs_glob_mesh->n_cells;
  const cs_lnum_t n_cells_ext = cs_glob_mesh->n_cells_with_ghosts;
  const cs_real_t *volume = cs_glob_mesh_quantities->cell_vol;
  const int keysca = cs_field_key_id("scalar_id");
  cs_lnum_t iel, iel1, npoint;
  cs_real_t xabs, xu, xcent, xc, xmom, xcm, xyz[3];
  int irangv, irang1;
  cs_field_t *sca;

  for (int f_id = 0; f_id < cs_field_n_fields(); f_id++) {
    cs_field_t *f = cs_field_by_id(f_id);
    if (cs_field_get_key_int(f, keysca) == 1) {
      sca = f;
      break;
    }
  }

  if (cs_glob_time_step->nt_cur == cs_glob_time_step->nt_max) {

    for (int iprofi = 0 ; iprofi < 3 ; iprofi++) {
      if (cs_glob_rank_id <= 0) {
        if (iprofi == 0)
          file = fopen("profile_1.dat","w");
        else if (iprofi == 1)
          file = fopen("profile_2.dat","w");
        else if (iprofi == 2)
          file = fopen("profile_3.dat","w");
        fprintf(file,"# z(m) U(m/s) Scal Scal mean\n");
      }

      cs_field_t *t_moment = cs_time_moment_get_field(0);

      npoint = 200;
      iel1 = -999;
      irang1 = -999;

      /* X coordinate for the injection */
      xcent = 0.043;
      /* Profile 1 : plan de l'injection */
      /* Profile 2 : plan à 4mm de l'injection */
      /* Profile 3 : plan à 8mm de l'injection */
      for (cs_lnum_t ii = 0 ; ii < npoint ; ii++) {
        if (iprofi == 0) {
          xyz[0] = xcent;
          xyz[2] = 0.015 + (cs_real_t)ii/(cs_real_t)(npoint-1)*0.06;
        } else if (iprofi == 1) {
          xyz[0] = xcent + 0.04;
          xyz[2] = (cs_real_t)ii/(cs_real_t)(npoint-1)*0.075;
        } else if (iprofi == 2) {
          xyz[0] = xcent + 0.08;
          xyz[2] = (cs_real_t)ii/(cs_real_t)(npoint-1)*0.075;
        }

        xyz[1] = 0.;

        CS_PROCF(findpt, FINDPT)(&n_cells_ext, &n_cells, (cs_real_t *)cell_cen,
                                 &xyz[0], &xyz[1], &xyz[2],
                                 &iel,    &irangv);

        if (iel != iel1 || irangv != irang1) {
          iel1 = iel;
          irang1 = irangv;

          /* Fill temporary variables xu, xvn ...
           * for the process that owns the point
           * and broadcast data to other ranks. */
          if (cs_glob_rank_id == irangv) {
            iel--;
            xabs = cell_cen[iel][2];
            xu = sqrt(pow(vel[iel][0],2)
                     +pow(vel[iel][1],2)
                     +pow(vel[iel][2],2));
            xc = 0.04401*sca->val[iel]/
                 ((0.04401 - 0.0040026)*sca->val[iel] + 0.0040026);
            xmom = t_moment->val[iel];
            xcm = 0.04401*xmom/((0.04401 - 0.0040026)*xmom + 0.0040026);
          } else {
            xabs = 0.;
            xu = 0.;
            xc = 0.;
            xcm = 0.;
          }

          /* Broadcast to other ranks in parallel case */
          if (cs_glob_rank_id >= 0) {
            cs_parall_bcast(irangv, 1, CS_DOUBLE, &xabs);
            cs_parall_bcast(irangv, 1, CS_DOUBLE, &xu);
            cs_parall_bcast(irangv, 1, CS_DOUBLE, &xc);
            cs_parall_bcast(irangv, 1, CS_DOUBLE, &xcm);
          }

          if (cs_glob_rank_id <= 0)
            fprintf(file, "%.6e %.6e %.6e %.6e\n",xabs, xu, xc, xcm);
        }
      }
      if (cs_glob_rank_id <= 0)
        fclose(file);
    }
  }
}

END_C_DECLS
